@@ -69,6 +69,8 @@ pub struct SynthMinidump {
     linux_proc_status: Option<SimpleStream>,
     /// /proc/self/limits string
     linux_proc_limits: Option<SimpleStream>,
+    /// Soft errors JSON string
+    soft_errors: Option<SimpleStream>,
     /// Continuous memory used by `Memory64List` stream
     memory64_section: Option<Section>,
     /// List of handles in this minidump.
@@ -228,6 +230,7 @@ impl SynthMinidump {
             linux_cpu_info: None,
             linux_proc_status: None,
             linux_proc_limits: None,
+            soft_errors: None,
             crashpad_info: None,
             memory64_section: Some(memory64_section),
             handle_data_stream: Some(ExListStream::new_with_header_size(
@@ -391,6 +394,15 @@ impl SynthMinidump {
         self
     }
 
+    /// Set the contents of the `MozSoftErrors` stream.
+    pub fn set_soft_errors(mut self, soft_errors: &str) -> SynthMinidump {
+        self.soft_errors = Some(SimpleStream {
+            stream_type: md::MINIDUMP_STREAM_TYPE::MozSoftErrors as u32,
+            section: Section::new().append_bytes(soft_errors.as_bytes()),
+        });
+        self
+    }
+
     /// Set the contents of the `LinuxCpuInfo` stream.
     pub fn set_linux_cpu_info(mut self, cpu_info: &[u8]) -> SynthMinidump {
         self.linux_cpu_info = Some(SimpleStream {
@@ -493,6 +505,9 @@ impl SynthMinidump {
         if let Some(stream) = self.linux_proc_limits.take() {
             self = self.add_stream(stream);
         }
+        if let Some(stream) = self.soft_errors.take() {
+            self = self.add_stream(stream);
+        }
         if let Some(stream) = self.linux_environ.take() {
             self = self.add_stream(stream);
         }
