@@ -13,10 +13,8 @@ mod arm64_old;
 mod mips;
 pub mod symbols;
 pub mod system_info;
-mod unwind;
 mod x86;
 
-use self::unwind::Unwind;
 use minidump::*;
 use minidump_common::utils::basename;
 use scroll::ctx::{SizeWith, TryFromCtx};
@@ -29,9 +27,25 @@ use tracing::trace;
 pub use crate::symbols::*;
 pub use crate::system_info::*;
 
+#[derive(Clone, Copy)]
+struct GetCallerFrameArgs<'a, P> {
+    callee_frame: &'a StackFrame,
+    grand_callee_frame: Option<&'a StackFrame>,
+    stack_memory: UnifiedMemory<'a, 'a>,
+    modules: &'a MinidumpModuleList,
+    system_info: &'a SystemInfo,
+    symbol_provider: &'a P,
+}
+
+impl<P> GetCallerFrameArgs<'_, P> {
+    fn valid(&self) -> &MinidumpContextValidity {
+        &self.callee_frame.context.valid
+    }
+}
+
 mod impl_prelude {
     pub(crate) use super::{
-        unwind::Unwind, CfiStackWalker, FrameTrust, StackFrame, SymbolProvider, SystemInfo,
+        CfiStackWalker, FrameTrust, GetCallerFrameArgs, StackFrame, SymbolProvider,
     };
 }
 
@@ -547,9 +561,46 @@ struct CfiStackWalker<'a, C: CpuContext> {
     caller_ctx: C,
     caller_validity: HashSet<&'static str>,
 
+    module: &'a MinidumpModule,
     stack_memory: UnifiedMemory<'a, 'a>,
 }
 
+impl<'a, C> CfiStackWalker<'a, C>
+where
+    C: CpuContext + Clone,
+{
+    fn from_ctx_and_args<P, R>(
+        ctx: &'a C,
+        args: &'a GetCallerFrameArgs<'a, P>,
+        callee_forwarded_regs: R,
+    ) -> Option<Self>
+    where
+        R: Fn(&MinidumpContextValidity) -> HashSet<&'static str>,
+    {
+        let module = args
+            .modules
+            .module_at_address(args.callee_frame.instruction)?;
+        let grand_callee = args.grand_callee_frame;
+        Some(Self {
+            instruction: args.callee_frame.instruction,
+            has_grand_callee: grand_callee.is_some(),
+            grand_callee_parameter_size: grand_callee.and_then(|f| f.parameter_size).unwrap_or(0),
+
+            callee_ctx: ctx,
+            callee_validity: args.valid(),
+
+            // Default to forwarding all callee-saved regs verbatim.
+            // The CFI evaluator may clear or overwrite these values.
+            // The stack pointer and instruction pointer are not included.
+            caller_ctx: ctx.clone(),
+            caller_validity: callee_forwarded_regs(args.valid()),
+
+            module,
+            stack_memory: args.stack_memory,
+        })
+    }
+}
+
 impl<'a, C> FrameWalker for CfiStackWalker<'a, C>
 where
     C: CpuContext,
@@ -600,91 +651,26 @@ where
     }
 }
 
-#[tracing::instrument(name = "unwind_frame", level = "trace", skip_all, fields(idx = _frame_idx, fname = callee_frame.function_name.as_deref().unwrap_or("")))]
+#[tracing::instrument(name = "unwind_frame", level = "trace", skip_all, fields(idx = _frame_idx, fname = args.callee_frame.function_name.as_deref().unwrap_or("")))]
 async fn get_caller_frame<P>(
     _frame_idx: usize,
-    callee_frame: &StackFrame,
-    grand_callee_frame: Option<&StackFrame>,
-    stack_memory: Option<UnifiedMemory<'_, '_>>,
-    modules: &MinidumpModuleList,
-    system_info: &SystemInfo,
-    symbol_provider: &P,
+    args: &GetCallerFrameArgs<'_, P>,
 ) -> Option<StackFrame>
 where
     P: SymbolProvider + Sync,
 {
-    match callee_frame.context.raw {
+    match args.callee_frame.context.raw {
         /*
         MinidumpRawContext::PPC(ctx) => ctx.get_caller_frame(stack_memory),
         MinidumpRawContext::PPC64(ctx) => ctx.get_caller_frame(stack_memory),
         MinidumpRawContext::SPARC(ctx) => ctx.get_caller_frame(stack_memory),
          */
-        MinidumpRawContext::Arm(ref ctx) => {
-            ctx.get_caller_frame(
-                callee_frame,
-                grand_callee_frame,
-                stack_memory,
-                modules,
-                system_info,
-                symbol_provider,
-            )
-            .await
-        }
-        MinidumpRawContext::Arm64(ref ctx) => {
-            ctx.get_caller_frame(
-                callee_frame,
-                grand_callee_frame,
-                stack_memory,
-                modules,
-                system_info,
-                symbol_provider,
-            )
-            .await
-        }
-        MinidumpRawContext::OldArm64(ref ctx) => {
-            ctx.get_caller_frame(
-                callee_frame,
-                grand_callee_frame,
-                stack_memory,
-                modules,
-                system_info,
-                symbol_provider,
-            )
-            .await
-        }
-        MinidumpRawContext::Amd64(ref ctx) => {
-            ctx.get_caller_frame(
-                callee_frame,
-                grand_callee_frame,
-                stack_memory,
-                modules,
-                system_info,
-                symbol_provider,
-            )
-            .await
-        }
-        MinidumpRawContext::X86(ref ctx) => {
-            ctx.get_caller_frame(
-                callee_frame,
-                grand_callee_frame,
-                stack_memory,
-                modules,
-                system_info,
-                symbol_provider,
-            )
-            .await
-        }
-        MinidumpRawContext::Mips(ref ctx) => {
-            ctx.get_caller_frame(
-                callee_frame,
-                grand_callee_frame,
-                stack_memory,
-                modules,
-                system_info,
-                symbol_provider,
-            )
-            .await
-        }
+        MinidumpRawContext::Arm(ref ctx) => arm::get_caller_frame(ctx, args).await,
+        MinidumpRawContext::Arm64(ref ctx) => arm64::get_caller_frame(ctx, args).await,
+        MinidumpRawContext::OldArm64(ref ctx) => arm64_old::get_caller_frame(ctx, args).await,
+        MinidumpRawContext::Amd64(ref ctx) => amd64::get_caller_frame(ctx, args).await,
+        MinidumpRawContext::X86(ref ctx) => x86::get_caller_frame(ctx, args).await,
+        MinidumpRawContext::Mips(ref ctx) => mips::get_caller_frame(ctx, args).await,
         _ => None,
     }
 }
@@ -753,6 +739,13 @@ pub async fn walk_stack<P>(
         stack.thread_id,
         stack.thread_name.as_deref().unwrap_or(""),
     );
+
+    // All the unwinder code down below in `get_caller_frame` requires a valid `stack_memory`,
+    // where _valid_ means that we can actually read something from it. A call to `memory_range` will validate that,
+    // as it will reject empty stack memory or one with an overflowing `size`.
+    let stack_memory =
+        stack_memory.and_then(|stack_memory| stack_memory.memory_range().map(|_| stack_memory));
+
     // Begin with the context frame, and keep getting callers until there are no more.
     let mut has_new_frame = !stack.frames.is_empty();
     let mut on_walked_frame = on_walked_frame.into();
@@ -768,6 +761,10 @@ pub async fn walk_stack<P>(
             on_walked_frame(frame_idx, frame);
         }
 
+        let Some(stack_memory) = stack_memory else {
+            break;
+        };
+
         // Walk the new frame
         let callee_frame = &stack.frames.last().unwrap();
         let grand_callee_frame = stack
@@ -781,12 +778,14 @@ pub async fn walk_stack<P>(
         }
         let new_frame = get_caller_frame(
             frame_idx,
-            callee_frame,
-            grand_callee_frame,
-            stack_memory,
-            modules,
-            system_info,
-            symbol_provider,
+            &GetCallerFrameArgs {
+                callee_frame,
+                grand_callee_frame,
+                stack_memory,
+                modules,
+                system_info,
+                symbol_provider,
+            },
         )
         .await;
 
